package kohtaushallinta;

import java.util.ArrayList;
import java.util.List;

import javafx.collections.transformation.SortedList;

/**
 * @author Jussi Lipitsäinen
 * @version 3.3.2025
 * Tallennetaan tietueeseen liitettävät avainsanat
 */
public class Avainsanat {    
    private String[] sanat;
    private int lkm = 0;
    private int maxPituus = 25; //Yksittäisen sanan maksimipituus
    
    /**
     * Oletusarvoinen maksimi avainsanojan lukumäärälle, jota oletusmuodostin käyttää.
     */
    public final static int MAXLKM = 10;
    
  

    /** Muodostin
     * @param maxLkm Avainsanojen maksimimäärä
     * @param sanat Tallennettavat sanat
     * @param erotin erotinmerkki
     * @param maxPituus Sanan maksimipituus
     */
    public Avainsanat(int maxLkm, int maxPituus, String sanat, char erotin) {
        if (maxLkm < 0) throw new IllegalArgumentException("Avainsanojen lukumäärä ei voi olla negatiivinen: " + maxLkm);
        this.sanat = new String[maxLkm];
        this.lkm = 0;
        this.maxPituus = maxPituus;
        this.lisaaJonosta(sanat, erotin);
    }
    
    
    /**
     * Asettaa avainsanalistan maksimikoon ja Lukee merkkijonosta avainsanat eroteltuna erotinmerkillä.
     * Sanan maksimipituuden ylittävät merkit jätetään lukematta, kuten myös sanat, jotka ylittävät maksimimäärän.
     * @param maxLkm Avainsanataulukon koko
     * @param sanat avainsanat erotinmerkillä eroteltuina
     * @param erotin käytetty erotinmerkki
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(5, "   Yksi Kaksi Kolme    Neljä Viisi    Kuusi", ' ');
     *  a.getLkm() === 5;
     *  a.get(1) === "Kaksi";
     *  a.get(4) === "Viisi";
     *  a = new Avainsanat(6, "Yksi, Kaksi , Kolme   , Neljä,   ");
     *  a.getLkm() === 4;
     * </pre>
     */
    public Avainsanat(int maxLkm, String sanat, char erotin) {
        if (maxLkm < 0) throw new IllegalArgumentException("Avainsanojen lukumäärä ei voi olla negatiivinen: " + maxLkm);
        this.sanat = new String[maxLkm];
        this.lkm = 0;
        this.lisaaJonosta(sanat, erotin);
    }
    
    
    /**
     * Oletus
     */
    public Avainsanat() {
        this.sanat = new String[MAXLKM];
        this.lkm = 0;
    }
    
    
    /**
     * Luo avainsanat taulukosta. Jos tarjottu sana on pidempi kuin maxPituus, huomioidaan vain alkuosa rajaan asti.
     * Jos parametritaulukko on pidempi kuin sanojen maksimimäärä, tallennetaan sanoja vain maksimimäärään asti. 
     * @param avainsanat Tallennettavat sanat
     * @example
     * <pre name="test">
     *  String[] sanat = {};
     *  Avainsanat as = new Avainsanat(sanat);
     *  as.toString() === "";
     *  sanat = new String[] {"Sana1", "Sana2jokaonihanliianpitkätähän"};
     *  as = new Avainsanat(sanat);
     *  as.toString() === "Sana1, Sana2jokaonihanliianpitkä";
     *  sanat = new String[] {"Sana1", "Sana2jokaonihanliianpitkätähän", "Sana2jokaonihanliianpitkä", "Sana3", "", "Sana4" };
     *  as = new Avainsanat(sanat);
     *  as.toString() === "Sana1, Sana2jokaonihanliianpitkä, Sana3, Sana4";
     *  sanat = new String[] {"S1", "S2", "S3", "S4", "S5", "S6", "S7", "S8", "S9", "S10", "S11", "S12", "S13"};
     *  as = new Avainsanat(sanat);
     *  as.toString() === "S1, S2, S3, S4, S5, S6, S7, S8, S9, S10";
     * </pre>
     */
    public Avainsanat(String[] avainsanat) {
        this();
        for (String s : avainsanat)
            if (s != null) this.lisaa(s); 
    }
    
    
    /**
     * @param sanat merkkijono, jossa on lisättävät sanat
     * @param erotin merkkijonossa käytetty erotin
     */
    public Avainsanat(String sanat, char erotin) {
        this(MAXLKM, sanat, erotin);
    }
    
    
    /**
     * Luo MAXLKM kokoisen Avainsanat merkkijonosta, jossa sanat on eroteltu merkillä ','.
     * Poistaa avainsanojen alusta ja lopusta tyhjät merkit.
     * @param sanat merkkijono
     */
    public Avainsanat(String sanat) {
        this(MAXLKM, sanat, ',');
    }
    
    
    /**
     * Luo Avainsanat merkkijonosta, jossa sanat on eroteltu merkillä ','.
     * Poistaa avainsanojen alusta ja lopusta tyhjät merkit.
     * @param maxLkm Kuinka monta avainsanaa mahtuu
     * @param sanat merkkijono
     */
    public Avainsanat(int maxLkm, String sanat) {
        this(maxLkm, sanat, ',');
    }
    
    
    /**
     * Tarkistaa, montako sanaa on tallennettuna
     * @return tallennettujen sanojen lkm
     */
    public int getLkm() {
        return lkm;        
    }
            
    
    /** Asettaa avainsanan maksimipituuden
     * @param max maksimipituus
     */
    public void setMaxPituus(int max) {
        if (max < 0) throw new IllegalArgumentException("Sanan maksimipituus ei voi olla negatiivinen: " + max);
        this.maxPituus = max;
    }
    
    
    /**
     * Hakee yksittäisen avainsanan maksimipituuden
     * @return avainsanan maksimipituus
     */
    public int getMaxPituus() {
        return this.maxPituus;
    }
    
        
    /** Hakee avainsanojen maksimimäärän
     * @return avainsanojen maksimimäärän
     */
    public int getMaxLkm() {
        return this.sanat.length;
    }
    
    
    /** Hakee avainsanan indeksistä. Palauttaa tyhjän merkkijonon, jos indeksi on kelvollinen, mutta siinä ei ole avainsanaa tallennettuna.
     * @param indeksi haettava indeksi
     * @return indeksissä olevan avainsanan
     * @example
     * <pre name="test">
     *  Avainsanat as = new Avainsanat(10, "sana1, sana2,sana3");
     *  as.get(0) === "sana1";
     *  as.get(9) === "";
     *  as.get(1) === "sana2";
     *  as.get(2) === "sana3";
     *  as.get(4) === "";
     *  try {
     *      as.get(11);
     *  } catch (IndexOutOfBoundsException ie) {
     *      ie.toString() === "java.lang.IndexOutOfBoundsException: Avainsanaindeksi on oltava väliltä 0-9. Haettu indeksi: 11";
     *  }
     *  try {
     *      as.get(-1);
     *  } catch (IndexOutOfBoundsException ie) {
     *      ie.toString() === "java.lang.IndexOutOfBoundsException: Avainsanaindeksi on oltava väliltä 0-9. Haettu indeksi: -1";
     *  }
     * </pre>
     */
    public String get(int indeksi) {
        if (indeksi < 0 || indeksi >= this.getMaxLkm()) throw new IndexOutOfBoundsException("Avainsanaindeksi on oltava väliltä 0-"
                                                                                                + (this.getMaxLkm()-1) +
                                                                                                ". Haettu indeksi: " + indeksi);
        if (indeksi < this.lkm) return this.sanat[indeksi];
        return "";
    }
    
    
    /**
     * Lisää listaan avainsanat, joita siinä ei jo ole
     * @param lista lista, johon lisätään
     * @example
     * <pre name="test">
     * 
     * </pre>
     */
    public void lisaaListaan(List<String> lista) {
        for (String sana : this.sanat)
            if (!lista.contains(sana)) lista.add(sana);        
    }
    
    
    /**
     * Lisää sanan, jos on tilaa
     * @param sana lisättävä sana
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(6, "Yksi, Kaksi , Kolme, Neljä, Viisi");
     *  a.lisaa("Kolme");
     *  a.toString() === "Yksi, Kaksi, Kolme, Neljä, Viisi";
     *  try {
     *      a.lisaa("Kuusi");
     *  } catch (IllegalStateException ie) {
     *      ie.toString() === "java.lang.IllegalStateException: Ei tilaa lisätä uutta sanaa: Kuusi";
     *  }
     *  a.toString() === "Yksi, Kaksi, Kolme, Neljä, Viisi, Kuusi";
     *  a = new Avainsanat();
     *  a.lisaa("123456789012345678901234567890");
     *  a.toString() === "1234567890123456789012345";
     * </pre>
     */
    public final void lisaa(String sana) {
        if (this.lkm >= this.sanat.length) throw new IllegalStateException("Ei tilaa lisätä uutta sanaa: " + sana);
        String s = sana.strip();
        if (s.length() < 1) return;
        if (s.length() > this.maxPituus) s = s.substring(0, maxPituus);
        if (hae(s) == -1)
            this.sanat[this.lkm++] = s;
    }
    
    
    /**
     * Lisää avainsanat jonosta eroteltuina erotinmerkillä. Lopettaa lisäämisen, jos tila täyttyy.
     * @param jono merkkijono
     * @param erotin erotinmerkki
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat();  
     *  a.lisaaJonosta ("Yksi, Kaksi, Kolme", ',');
     *  a.toString() === "Yksi, Kaksi, Kolme";
     *  a.lisaaJonosta ("Neljä Viisi Kuusi Seitsemän", ' ');
     *  a.toString() === "Yksi, Kaksi, Kolme, Neljä, Viisi, Kuusi, Seitsemän";
     *  try {
     *      a.lisaaJonosta("Kahdeksan # Yhdeksän # Kymmenen # Yksitoista # Kaksitoista", '#');
     *  } catch (IllegalStateException ie) {
     *      ie.toString() === "java.lang.IllegalStateException: Ei tilaa lisätä uutta sanaa: Yksitoista";
     *  }
     *  a.toString() === "Yksi, Kaksi, Kolme, Neljä, Viisi, Kuusi, Seitsemän, Kahdeksan, Yhdeksän, Kymmenen";
     * </pre>
     */
    public final void lisaaJonosta(String jono, char erotin) {
        String[] uusi = jono.split("[" + erotin +"]");
        for (String s : uusi) lisaa(s);
       
    }
    
    
    /**
     * Lisää jonosta käyttäen oletuserotinta ','
     * @param jono lisättävä jono
     */
    public final void lisaaJonosta(String jono) {
        this.lisaaJonosta(jono, ',');
    }
    
    /**
     * Jos sana löytyy taulukosta, poistaa sen. Siirtää taulukon viimeisen alkion poistettavan paikalle
     * ja määrittää viimeisen alkion tyhjäksi. Päivittää sanojen lukumäärän. 
     * @param sana poistettava sana
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(6, "Yksi, Kaksi , Kolme, Neljä, Viisi");
     *  a.poista("Kolme");
     *  a.toString() === "Yksi, Kaksi, Viisi, Neljä";
     *  a.poista("Kolme");
     *  a.toString() === "Yksi, Kaksi, Viisi, Neljä";
     * </pre>
     */
    public void poista(String sana) {
        int i = hae(sana);
        if (i > -1) poista(i);        
    }
    
    
    /**
     * Jos annetussa indeksissä on sana, poistetaan se. Siirtää taulukon viimeisen alkion poistettavan paikalle
     * ja määrittää viimeisen alkion tyhjäksi. Päivittää sanojen lukumäärän. 
     * @param indeksi indeksi, josta poistetaan.
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(6, "Yksi, Kaksi , Kolme, Neljä, Viisi");
     *  a.poista(0);
     *  a.toString() === "Viisi, Kaksi, Kolme, Neljä";
     *  a.poista(3);
     *  a.toString() === "Viisi, Kaksi, Kolme";
     *  a.poista(3);
     *  try {
     *      a.poista(-1);
     *  } catch (IndexOutOfBoundsException ie) {
     *      ie.toString() === "java.lang.IndexOutOfBoundsException: Avainsanaindeksi on oltava väliltä 0-5. Haettu indeksi: -1";
     *  }
     *  try {
     *      a.poista(7);
     *  } catch (IndexOutOfBoundsException ie) {
     *      ie.toString() === "java.lang.IndexOutOfBoundsException: Avainsanaindeksi on oltava väliltä 0-5. Haettu indeksi: 7";
     *  }   
     *  a.toString() === "Viisi, Kaksi, Kolme";
     *  a.poista(0);
     *  a.poista(0);
     *  a.poista(0);
     *  a.poista(0);
     *  a.poista(0);
     *  a.toString() === "";
     * </pre>
     */
    public void poista(int indeksi) {
        if (indeksi < 0 || indeksi >= this.getMaxLkm()) throw new IndexOutOfBoundsException("Avainsanaindeksi on oltava väliltä 0-"
                                                                                         + (this.getMaxLkm()-1) +
                                                                                         ". Haettu indeksi: " + indeksi);
        if (indeksi >= this.getLkm()) return;
        this.sanat[indeksi] = this.sanat[this.lkm-1];
        this.sanat[this.lkm-1] = null;
        this.lkm--;
    }
    
    
    /**
     * Hakee sanan indeksin
     * @param sana haettava sana
     * @return Sanan indeksin. Jos ei löydy, -1.
     * @example
     * <pre name="test">
     *  Avainsanat a = new Avainsanat(6, "Yksi, Kaksi , Kolme, Neljä, Viisi");
     *  a.hae("Kolme") === 2;
     *  a.hae("fvbsbfs") === -1;
     *  a = new Avainsanat();
     *  a.hae("Yksi") === -1;
     * </pre>
     */
    public int hae(String sana) {        
        for (int i = 0; i < this.lkm; i++)
            if (sana.equals(this.sanat[i])) return i;
        return -1;
    }
    
    
    
    /** Hakee avainsanat taulukkona
     * @return merkkijonotaulukon, jossa tallennetut avainsanat
     * @example
     * <pre name="test">
     *  Avainsanat as;
     *  String[] tulos;
     *  as = new Avainsanat(10, $avainsanat);
     *  tulos = as.getSanat();
     *  tulos[0] === $1;
     *  tulos[1] === $2;
     *  tulos[2] === $3;
     *  tulos[3] === $4;
     *  
     *  $avainsanat                     | $s1     | $s2     | $s3     | $s4
     *  ----------------------------------------------------------------------
     *  "sana1, sana2, sana3, sana4"    | "sana1" | "sana2" | "sana3" | "sana4"
     *  ""                              | null    | null    | null    | null
     *  "abcdef"                        | "abcdef"| null    | null    | null
     * </pre>
     */
    public String[] getSanat() {
        String[] uusi = new String[this.lkm];
        for (int i = 0; i < lkm; i++) uusi[i] = this.sanat[i];
        return uusi;
    }
    
    
    /**
     * Lukee avainsanat jonosta. Mahdolliset vanhat sanat poistetaan.
     * @param jono merkkijono
     * @example
     * <pre name="test">
     *  Avainsanat as = new Avainsanat(5, "s1, s2, s3, s4, s5");
     *  as.lueJonosta("uusi sana");
     *  as.toString() === "uusi sana";
     *  try {
     *      as.lueJonosta("sana1, sana2, sana3, sana4, sana5, sana6");
     *  } catch (IllegalStateException ie) {
     *      ie.getMessage() === "Ei tilaa lisätä uutta sanaa: sana6";
     *  }
     *  as.toString() === "sana1, sana2, sana3, sana4, sana5";
     *  as.lueJonosta("");
     *  as.toString() === "";
     * </pre>
     */
    public void lueJonosta(String jono) {
        this.sanat = new String[sanat.length];
        this.lkm = 0;
        this.lisaaJonosta(jono);
    }
    
    
    /**
     * Palauttaa avainsanat listana
     * @return merkkijonolistan
     */
    public SortedList<String> getList() {
        /*
        SortedList<String> al = new SortedList<String>(al);
        for (String sana : this.sanat)
            if (sana != null && sana.length() > 0) al.add(sana);
        return al;
        */
        return null;
    }
    
    
    @Override
    public String toString() {
        if (this.lkm == 0) return "";
        String jono = this.sanat[0];
        if (this.lkm > 1)
            for (int i = 1; i < this.lkm; i++) jono += ", " + this.sanat[i];                        
        return jono;
    }
}
