package kohtaushallinta;

import java.util.ArrayList;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author Jussi Lipitsäinen
 * @version 19.3.2025
 * Luokka itse kohtausta varten. Tallentaa kohtauksen tiedot ja siihen linkitetyt tietueet.
 */
public class Kohtaus extends Tietue {
    private ArrayList<Tietue> hahmot = new ArrayList<Tietue>();
    private ArrayList<Tietue> asiat = new ArrayList<Tietue>();
    private String muistiinpanot = "";
    
    /** Käsitellään linkitettyjä hahmoja */
    public static final int HAHMOT = 0;
    /** Käsitellään linkitettyjä asioita */
    public static final int ASIAT = 1;
    
    
    /**
     * Luo uuden Kohtauksen ID:n ja otsikon avulla
     * @param id ID
     * @param otsikko otsikko
     */
    public Kohtaus(int id, String otsikko) {
        super(id, otsikko);
    }
    
    
    /**
     * Luo uden Kohtauksen
     * @param id ID
     * @param otsikko otsikko
     * @param avainsanat avainsanat
     * @param kuvaus kuvaus
     */
    public Kohtaus(int id, String otsikko, Avainsanat avainsanat, String kuvaus) {
        super(id, otsikko, avainsanat, kuvaus);
    }
    
    
    /**
     * Asettaa muistiinpanot-kentän sisällön
     * @param jono muistiinpanot
     */
    public void setMuistiinpanot(String jono) {
        this.muistiinpanot = jono;
    }
    
    
    /**
     * Hakee muistiinpanot-kentän sisällön
     * @return muistiinpanot
     */
    public String getMuistiinpanot() {
        return this.muistiinpanot;
    }
    
    
    /**
     * Asettaa halutun linkkilistan sisällöksi taulukon tietueet.
     * Tyhjentää listan sisällön ennen operaatiota.
     * @param tyyppi haluttu tyyppi (HAHMO/ASIA)
     * @param tietueet asettettavat tietueet.
     * @throws IllegalArgumentException listan tyyppi ei ole kelvollinen
     * @example
     * <pre name="test">
     *  Tietue[] tt1 = { new Tietue(0, "tietue0"), new Tietue(1, "tietue1"), new Tietue(2, "tietue2") };
     *  Kohtaus k = new Kohtaus(0, "kohtaus");
     *  k.setLinkit(Kohtaus.HAHMOT, tt1);
     *  try {
     *      k.setLinkit(2, tt1);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): 2 ei kelpaa";
     *  }
     *  try {
     *      k.setLinkit(-1, tt1);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): -1 ei kelpaa";
     *  }
     * </pre>
     * 
     */
    public void setLinkit(int tyyppi, Tietue[] tietueet) throws IllegalArgumentException {
        ArrayList<Tietue> lista = valitseLista(tyyppi);
        for (Tietue t : tietueet) lista.add(t);
    }
    
    
    /**
     * Hakee linkitettyjen tiedostojen ID:t
     * @param tyyppi HAHMO (0) tai ASIA (1)
     * @return linkitettyjen tiedostojen ID:t kokonaislukutaulukkona
     * @throws IllegalArgumentException listan tyyppi ei ole kelvollinen
     * @example
     * <pre name="test">
     *  Kohtaus k = new Kohtaus(0, "kohtaus");
     *  k.getLinkkiIDt(Kohtaus.ASIAT).length === 0;
     *  k.getLinkkiIDt(Kohtaus.HAHMOT).length === 0;
     *  try {
     *      k.getLinkkiIDt(2);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): 2 ei kelpaa";
     *  }
     *  try {
     *      k.getLinkkiIDt(-1);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): -1 ei kelpaa";
     *  }
     *  
     *  k.linkita(Kohtaus.ASIAT, new Tietue(1, "asia1"));
     *  k.linkita(Kohtaus.ASIAT, new Tietue(3, "asia3"));
     *  k.linkita(Kohtaus.ASIAT, new Tietue(4, "asia4"));
     *  k.linkita(Kohtaus.ASIAT, new Tietue(9, "asia9"));
     *  k.linkita(Kohtaus.HAHMOT, new Tietue(2, "hahmo2"));
     *  k.linkita(Kohtaus.HAHMOT, new Tietue(3, "hahmo3"));
     *  k.linkita(Kohtaus.HAHMOT, new Tietue(4, "hahmo4"));
     *  int[] tarkistush = { 2, 3, 4 };
     *  int[] tarkistusa = { 1, 3, 4, 9 };
     *  int[] asiat = k.getLinkkiIDt(Kohtaus.ASIAT);
     *  int[] hahmot = k.getLinkkiIDt(Kohtaus.HAHMOT);
     *  for (int i = 0; i < 4; i++)
     *      asiat[i] === tarkistusa[i];
     *  for (int j = 0; j < 3; j++)
     *      hahmot[j] === tarkistush[j];
     *  try {
     *      k.getLinkkiIDt(2);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): 2 ei kelpaa";
     *  }
     *  try {
     *      k.getLinkkiIDt(-1);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): -1 ei kelpaa";
     *  }
     * </pre>
     */
    public int[] getLinkkiIDt(int tyyppi) throws IllegalArgumentException {
        ArrayList<Tietue> lista = valitseLista(tyyppi);
        int[] idt = new int[lista.size()];
        int i = 0;
        for (Tietue t : lista)
            if (t != null) idt[i++] = t.getID(); 
        return idt;
    }
    
    /**
     * Valitsee listan annetun tyypin perusteella
     * @param tyyppi HAHMO (0) tai ASIA (1)
     * @return kyseisen listan
     * @throws IllegalArgumentException listan tyyppi ei ole kelvollinen
     */
    private ArrayList<Tietue> valitseLista (int tyyppi) throws IllegalArgumentException {        
        switch (tyyppi) {
        case HAHMOT:
            return this.hahmot;            
        case ASIAT:
            return this.asiat;
        default:
            throw new IllegalArgumentException("Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): " + tyyppi + " ei kelpaa");
        }
    }
    
    
    /**
     * Jos samalla id:llä ei ole tietuetta halutussa listassa, lisää sen
     * @param tyyppi HAHMO/ASIA
     * @param t linkitettävä tietue
     * @example
     * <pre name="test">
     *  Kohtaus k = new Kohtaus (0, "kohtaus");
     *  k.linkita(Kohtaus.ASIAT, new Tietue(2, "asia2"));
     *  k.linkita(Kohtaus.ASIAT, new Tietue(2, "uusi asia"));
     *  int[] idt = k.getLinkkiIDt(Kohtaus.ASIAT);
     *  idt.length === 1;
     *  idt[0] === 2;
     *  k.get
     *  k.linkita(Kohtaus.HAHMOT, new Tietue(1, "hahmo1"));    
     * </pre>
     */
    public void linkita(int tyyppi, Tietue t) {        
        if (this.onkoLinkitetty(tyyppi, t.getID())) return;
        ArrayList<Tietue> lista = valitseLista(tyyppi);
        lista.add(t);
    }
    
    
    /**
     * Tarkistaa, onko samalla ID:llä tietue listassa
     * @param tyyppi HAHMOT (0) / ASIAT (1)
     * @param id haettava ID
     * @return true, jos löytyy. Muuten false
     * @throws IllegalArgumentException listan tyyppi ei ole kelvollinen
     * @example
     * <pre name="test">
     *  Kohtaus k = new Kohtaus(0, "Kohtaus");
     *  k.linkita(Kohtaus.HAHMOT, new Tietue(3, "hahmo3"));
     *  k.onkoLinkitetty(Kohtaus.HAHMOT, 3) === true;
     *  k.onkoLinkitetty(Kohtaus.HAHMOT, 2) === false;
     *  k.onkoLinkitetty(Kohtaus.ASIAT, 3) === false;
     *  k.linkita(Kohtaus.ASIAT, new Tietue(0, "asia0"));
     *  k.onkoLinkitetty(Kohtaus.ASIAT, 0) === true;
     *  try {
     *      k.onkoLinkitetty(2, 3);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): 2 ei kelpaa";
     *  }
     *  try {
     *      k.onkoLinkitetty(-1, 3);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Listan tyypin on oltava HAHMOT (0) tai ASIAT (1): -1 ei kelpaa";
     *  }
     * </pre>
     */
    public boolean onkoLinkitetty(int tyyppi, int id)  throws IllegalArgumentException {
        ArrayList<Tietue> lista = valitseLista(tyyppi);
        for(Tietue t : lista) if (t.getID() == id) return true;
        return false;
    }

    
    
    /**
     * Palauttaa Kohtauksen syötetyn merkkijonon perusteella
     * @param jono merkkijono
     * @param erotin kenttiä erottava merkki
     * @param kenttia montako kenttää luetaan merkkijonosta
     * @return luodun Kohtauksen
     * @throws NumberFormatException ekaa kenttää ei saatu luettua ID:ksi
     * @throws IllegalArgumentException otsikkokenttää ei ole määritelty
     * @example
     * <pre name="test">
     *  
     * </pre>
     */
    public static Kohtaus lueJono(String jono, char erotin, int kenttia) throws NumberFormatException, IllegalArgumentException {
        StringBuilder sb = new StringBuilder(jono);
        int k = kenttia < 2 ? 2 : kenttia;
        Tietue t = Tietue.lueSB(sb, '|', k);
        Kohtaus uusi = new Kohtaus(t.getID(), t.getOtsikko(), t.getAvainsanat(), t.getKuvaus());
        if (kenttia > 4) uusi.setMuistiinpanot(Mjonot.erota(sb, '|').strip());
        return uusi;
    }

    
    public static Kohtaus lueTaulukko(String[] taulukko) throws IllegalArgumentException, NumberFormatException {
        if (taulukko.length < 2) {
            throw new IllegalArgumentException("ID ja otsikko on oltava määritelty.");
        }
        int id = Integer.parseInt(taulukko[0].strip());
        Kohtaus uusi = new Kohtaus(id, taulukko[1].strip());
        if (taulukko.length < 3) return uusi;
        uusi.getAvainsanat().lueJonosta(taulukko[2].strip());
        if (taulukko.length < 4) return uusi;
        uusi.setKuvaus(taulukko[3].strip());
        if (taulukko.length < 5) return uusi;
        uusi.setMuistiinpanot(taulukko[4].strip());
        return uusi;
    }
    
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.toString());
        sb.append("\nHahmot:");
        for (Tietue t : this.hahmot) sb.append(" " + t.toString());
        sb.append("\nAsiat:");
        for (Tietue t : this.asiat) sb.append(" " + t.toString());
        return sb.toString();
    }


}
