package kohtaushallinta;

import java.io.IOException;

/**
 * @author Jussi Lipitsäinen
 * @version 21.3.2025
 * Hallinnoi kokonaisuuden toimintaa ja pitää huolta linkitysten eheydestä.
 */
public class Kohtaushallinta {
    private Kohtaukset kohtaustietueet;
    private Tietueet hahmotietueet;
    private Tietueet asiatietueet;
    private Linkkihallinta linkit;
    private String kohtausTdst = "kohtaukset.dat";
    private String hahmoTdst = "hahmot.dat";
    private String asiaTdst = "asiat.dat";
    private String asialinkit = "kohtauksenAsiat.dat";
    private String hahmolinkit = "kohtauksenHahmot.dat";
    
    public static final int HAHMOT = 0;
    public static final int ASIAT = 1;
    public static final int KOHTAUKSET = 2;
    public static final int LINKIT = 3;
    private static final int LKM = 200;
    
    
    
    /**
     * Oletusmuodostin
     */
    public Kohtaushallinta() {
        this.kohtaustietueet = new Kohtaukset(LKM, kohtausTdst);
        this.hahmotietueet = new Tietueet(LKM, hahmoTdst);
        this.asiatietueet = new Tietueet(LKM, asiaTdst);
        this.linkit = new Linkkihallinta(LKM, hahmolinkit, asialinkit);
    }
    
    
    public Kohtaushallinta(int lkm) {
        this.kohtaustietueet = new Kohtaukset(LKM, kohtausTdst);
        this.hahmotietueet = new Tietueet(lkm, hahmoTdst);
        this.asiatietueet = new Tietueet(lkm, asiaTdst);
        this.linkit = new Linkkihallinta(lkm, hahmolinkit, asialinkit);
    }
    
    
    /**
     * Asettaa tiedostonimet ja muodostaa Kohtaushallinnan
     * @param kohtauksetTdst kohtaustietueiden tiedostonimi
     * @param hahmotTdst hahmotietueiden tiedostonimi
     * @param asiatTdst asiatietueiden tiedostonimi
     */
    public Kohtaushallinta(String kohtauksetTdst, String hahmotTdst, String asiatTdst) {        
        this.kohtausTdst = kohtauksetTdst;
        this.hahmoTdst = hahmotTdst;
        this.asiaTdst = asiatTdst;
        
        this.kohtaustietueet = new Kohtaukset(LKM, kohtauksetTdst);
        this.hahmotietueet = new Tietueet(LKM, hahmotTdst);
        this.asiatietueet = new Tietueet(LKM, asiatTdst);
    }
    
    
    
    /**
     * Ladataan pyydettyjen tietueiden tiedot niiden tiedostosta.
     * @param tyyppi KOHTAUS (0), HAHMOT (1), ASIAT (2) TAI LINKIT (3)
     * @throws IllegalArgumentException tyyppi ei kelpaa
     * @throws IOException Tiedostoa ei löydy tai siitä ei voinut lukea
     */
    public void lataa(int tyyppi) throws IllegalArgumentException, IOException {
        switch(tyyppi) {
        case KOHTAUKSET:
            break;
        case HAHMOT:
            this.hahmotietueet.lataa();
            break;
        case ASIAT:
            this.asiatietueet.lataa();
            break;
        case LINKIT:
            this.linkit.lataa();
            break;
        default:
            throw new IllegalArgumentException("Metodin argumentin on oltava KOHTAUS (0), HAHMOT (1), ASIAT (2) TAI LINKIT (3): " + tyyppi + " ei kelpaa");
        }
    }
    
    
    /**
     * Lataa kaikki tietueet tiedostoista
     * @throws IllegalArgumentException yritetään ladata tyyppiä, jota ei ole määritelty
     * @throws IOException Tiedostoa ei löydy tai sitä ei voi lukea
     */
    public void lataa() throws IllegalArgumentException, IOException {
        for (int i = 0; i < 4; i++) lataa(i);
    }
    
    
    /**
     * Palauttaa Tietueen tyypin ja id:n peruseella 
     * @param tyyppi tyyppi: HAHMOT (0), ASIAT (1) tai KOHTAUKSET (2)
     * @param id haettava id
     * @return Tietueen, jos se löytyi. null, jos ei löydy.
     */
    public Tietue get(int tyyppi, int id) {
        switch(tyyppi) {
        case KOHTAUKSET:
            return kohtaustietueet.get(id);
        case HAHMOT:
            return hahmotietueet.get(id);
        case ASIAT:
            return asiatietueet.get(id);
        default:
            throw new IllegalArgumentException("Metodin argumentin on oltava KOHTAUS (0), HAHMOT (1) tai ASIAT: " + tyyppi + " ei kelpaa");
        }
    }
    

    /**
     * Luo uuden tietueen haluttua tyyppiä.
     * @param tyyppi HAHMOT (0) / ASIAT (1) / KOHTAUKSET (3)
     * @return Tietueen
     * @throws IllegalArgumentException argumentti ei viittaa mihinkään Tietueet-olioon
     * @example
     * <pre name="test">
     *  #THROWS IOException
     *  #import java.io.IOException;
     *  Kohtaushallinta kh = new Kohtaushallinta();
     *  kh.lataa();
     *  Tietue t = kh.uusiTietue(Kohtaushallinta.ASIAT);
     *  t.getID() === 6;
     *  t = kh.uusiTietue(Kohtaushallinta.HAHMOT);
     *  t.getID() === 5;
     *  t = (Tietue)kh.uusiTietue(Kohtaushallinta.KOHTAUKSET);
     *  t.getID() === 4;
     * </pre>
     */
    public Tietue uusiTietue(int tyyppi) throws IllegalArgumentException {
        return this.getTietueet(tyyppi).luoTietue();
    }
    
    
    /**
     * Hakee halutun Tietueet olion
     * @param tyyppi HAHMOT (0) / ASIAT (1) / KOHTAUKSET (3)
     * @return Tietueet
     * @throws IllegalArgumentException argumentti ei viittaa mihinkään Tietueet-olioon
     */
    private Tietueet getTietueet(int tyyppi) throws IllegalArgumentException {
        switch(tyyppi) {        
        case 0:
            return this.hahmotietueet;
        case 1:
            return this.asiatietueet;
        case 2:
            return this.kohtaustietueet;
        default:
            throw new IllegalArgumentException("Metodin argumentin on oltava KOHTAUS (0), HAHMOT (1) tai ASIAT: " + tyyppi + " ei kelpaa");
        }
    }
        
    
    //TODO: DEBUGGAUSTA VARTEN, POISTA
    @SuppressWarnings("javadoc")
    public static void main(String[] args) {
        /*
        Kohtaushallinta kh = new Kohtaushallinta(8);
        kh.lataa();
        kh.teeKohtaukset();
        kh.luoKohtaus("Uusi kohtaus");
        kh.luoKohtaus("Toinen uusi kohtaus", new String[] {"Sana1", "Sana2", "Sana3" }, "toisen kuvaus", "Kohtausmuistiinpanot");
        for (int i = 0; i < 8; i++) {
            System.out.println(kh.getKohtausString(i));
            System.out.println("-------------------------------");
        }
        */
    }
}
