package kohtaushallinta;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.ArrayList;
import java.util.Scanner;

import fi.jyu.mit.ohj2.VertaaTiedosto;

/**
 * @author Jussi Lipitsäinen
 * @version 7.3.2025
 * Tiedostoista lukemiseen ja niihin kirjoittamiseen liittyviä metodeja ja funktioita
 */
public class Tiedostokasittely {
    private static final String OLETUSHAKEMISTO = "tiedostot/";
    
    
    /**
     * Lukee tiedostosta tiedot Tietueisiin
     * @param kayttaja Tietueet
     * @param tiedostonimi luettava tiedosto
     * @throws IOException Lukeminen ei onnistu tai tiedostoa ei löydy
     * @example
     * <pre name="test">
     *  #THROWS IOException
     *  #import java.io.IOException;  
     *  #import fi.jyu.mit.ohj2.VertaaTiedosto;
     *  #import kohtaushallinta.Tietueet;
     *  String testi = "tiedostot/testi.dat";
     *  VertaaTiedosto.kirjoitaTiedosto(testi, "0 | otsikko1 | sana1, sana2 | kuvaus0\n" + 
     *                                          "4 | otsikko4 | sana1, sana3 | kuvaus4\n" + 
     *                                          "#5 | otsikko5 | sana1 | kuvaus5\n\n\n" +
     *                                          "2 | otsikko2 | sana3, sana4, sana1 | kuvaus2");
     *  Tietueet tt = new Tietueet(10, "testi.dat");
     *  lueTiedostostaStream(tt, "testi.dat");
     *  tt.get(0).toString() === "0 | otsikko1 | sana1, sana2 | kuvaus0";
     *  tt.get(4).toString() === "4 | otsikko4 | sana1, sana3 | kuvaus4";
     *  tt.get(2).toString() === "2 | otsikko2 | sana3, sana4, sana1 | kuvaus2";
     *  tt.get(5) === null;
     *  VertaaTiedosto.tuhoaTiedosto(testi);
     *  try {
     *      lueTiedostostaStream(tt, "testi.dat");
     *  } catch (IOException e) {
     *      e.getMessage() === "Tiedostoa tiedostot\\testi.dat ei löydy.";
     *  }
     *  VertaaTiedosto.kirjoitaTiedosto(testi, "");
     *  lueTiedostostaStream(tt, "testi.dat");
     *  tt.get(0) === null;
     *  tt.get(4) === null;
     *  tt.get(2) === null;
     *  VertaaTiedosto.tuhoaTiedosto(testi);
     * </pre>
     */
    public static void lueTiedostosta(TiedostoKayttaja kayttaja, String tiedostonimi) throws IOException {
        File tdst = new File(OLETUSHAKEMISTO + tiedostonimi);
        if (!tdst.exists()) throw new FileNotFoundException("Tiedostoa " + tdst + " ei löydy.");
        try (FileInputStream fs = new FileInputStream(tdst)) {
            kayttaja.lue(fs);
        }
    }
    
    
    /**
     * Avaa tiedoston tekstitiedostona ja lukee sen sisältämät rivit listaan.
     * @param lista ArrayList<String>, johon luetaan. Jos null, ei tehdä mitään.
     * @param tiedosto luettava tiedosto
     * @throws FileNotFoundException tiedostoa ei löydy
     */
    public static void lataaRivitListaan(ArrayList<String> lista, File tiedosto) throws FileNotFoundException {
        if (lista == null) return;
        try (Scanner s = new Scanner(new FileInputStream(tiedosto))) {
            while(s.hasNext())
                lista.add(s.nextLine().strip());
        }
    }    
   
    
    /**
     * Tallentaa Kayttäjän sisältämät tiedot tiedostoon
     * @param kayttaja tiedot sisältävä luokka 
     * @param tiedostonimi tiedosto, johon tallennetaan
     * @throws IOException Tiedostoa ei voinut tallentaa
     */
    public static void tallennaTiedostoon(TiedostoKayttaja kayttaja, String tiedostonimi) throws IOException {
        File tiedosto = new File(OLETUSHAKEMISTO + tiedostonimi);
        if (tiedosto.exists()) varmuuskopioi(tiedosto);
        try (FileWriter fw = new FileWriter(tiedosto)) {
            kayttaja.kirjoita(fw);
        }
    }
    
    
    /**
     * Tallentaa taulukon merkkijonoja tekstitiedoston riveiksi. Jos tiedosto on jo olemassa,
     * kopioi siitä varmuuskopion .bak -päätteellä korvaten mahdollisen edellisen varmuuskopion.
     * Jos taulukko on tyhjä, ei tee mitään.
     * @param taulukko tallennettava taulukko
     * @param tiedostonimi tiedosto, johon tallennetaan
     * @throws IOException tiedostoa ei voinut kirjoittaa
     * @example
     * <pre name="test">
     *  #THROWS IOException
     *  #import java.io.IOException;  
     *  #import fi.jyu.mit.ohj2.VertaaTiedosto;
     *  String tdst = "tiedostot/koe.txt";
     *  String testi = "tiedostot/testi.txt";
     *  String bak = testi + ".bak";
     *  String[] rivit;
     *  rivit = new String[] $rivit;
     *  VertaaTiedosto.kirjoitaTiedosto(tdst, $tdst);
     *  VertaaTiedosto.kirjoitaTiedosto(bak, "");
     *  tallennaTaulukko(rivit, "testi.txt");
     *  VertaaTiedosto.vertaaFileFile(tdst, testi) === null;
     *  VertaaTiedosto.kirjoitaTiedosto(tdst, $bak);
     *  VertaaTiedosto.vertaaFileFile(tdst, bak) === null;
     *  
     *  $rivit                         | $tdst              | $bak 
     *  ------------------------------------------------------------
     *  { "tt1", "tt2", "#tt3" }       | "tt1\ntt2\n#tt3"   | ""
     *  { "tt4", "tt5" }               | "tt4\ntt5"         | "tt1\ntt2\n#tt3"  
     *  { "", "", "", "" }             | "\n\n\n\n"         | "tt4\ntt5"
     *  { }                            | "\n\n\n\n"         | ""
     *  
     *  
     *  VertaaTiedosto.tuhoaTiedosto(tdst);
     *  VertaaTiedosto.tuhoaTiedosto(testi);
     *  VertaaTiedosto.tuhoaTiedosto(testi + ".bak");
     * </pre>
     * 
     */
    public static void tallennaTaulukko(String[] taulukko, String tiedostonimi) throws IOException {
        if (taulukko.length == 0) return;
        File tiedosto = new File(OLETUSHAKEMISTO + tiedostonimi);
        if (tiedosto.exists()) varmuuskopioi(tiedosto);
        try (FileWriter fw = new FileWriter(tiedosto)) {
            for(String s : taulukko)
                fw.write(s + "\n");
        }
    }
    
    
    /**
     * Tallentaa kopion tiedostosta samaan hakemistoon liittäen perään .bak-liitteen.
     * Korvaa mahdollisesti olemassa olevan saman nimisen tiedoston.
     * @param tiedosto kopioitava tiedosto
     * @throws IOException tiedostoa ei voi tallentaa
     * @example
     * <pre name="test">
     *  #THROWS IOException
     *  #import java.io.IOException;  
     *  #import fi.jyu.mit.ohj2.VertaaTiedosto;
     *  #import java.io.File;
     *  String tdst = "testi.dat";
     *  VertaaTiedosto.kirjoitaTiedosto(tdst, "\n\nTämä on testitiedosto\n#TEKSTIÄ\n\nLisäätekstiä\n\n\n");
     *  varmuuskopioi(new File(tdst));
     *  VertaaTiedosto.vertaaFileFile(tdst, tdst+".bak") === null;
     *  VertaaTiedosto.tuhoaTiedosto(tdst);
     *  VertaaTiedosto.tuhoaTiedosto(tdst+".bak");
     * </pre>
     */
    public static void varmuuskopioi(File tiedosto) throws IOException {                   
        Path polku = tiedosto.toPath();
        Files.copy(polku, polku.resolveSibling(polku.getFileName().toString() + ".bak" ), StandardCopyOption.REPLACE_EXISTING);
    }
}
