package kohtaushallinta;

import java.util.ArrayList;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author Jussi Lipitsäinen
 * @version 28.2.2025
 * Yleinen tietueluokka, jota käytetään hahmojen ja asioiden tallentamiseen. Ylläpitää niiden tietoja.
 */
public class Tietue {    
    private final int id;
    private String otsikko;
    private String kuvaus = "";
    private Avainsanat avainsanat = new Avainsanat();
    
    
    /**
     * Muodostaa uuden Tietueen
     * @param id ID
     * @param otsikko otsikko
     * @param avaimet Avainsanat
     * @param kuvaus kuvaus
     */
    public Tietue(int id, String otsikko, Avainsanat avaimet, String kuvaus) {
        this.id = id;
        this.otsikko = otsikko;
        this.avainsanat = avaimet;
        this.kuvaus = kuvaus;
    }
    
    
    /**
     * Muodostaa uuden tietueen, uusi Avainsanat ja tyhä kuvaus
     * @param id ID
     * @param otsikko otsikko
     */
    public Tietue (int id, String otsikko) {
        this(id, otsikko, new Avainsanat(), "");
    }
    
    
    /**
     * Muodostaa uuden Tietueen, kentät tyhjiä ID:tä lukuunottamatta
     * @param id ID
     */
    public Tietue(int id) {
        this(id, "", new Avainsanat(), "");
    }
    
    
    /**
     * Hakee ID:n
     * @return ID
     */
    public int getID() {
        return this.id;
    }
    
    
    /**
     * Hakee otsikko-kentän sisällön
     * @return otsikko
     */
    public String getOtsikko() {
        return this.otsikko;
    }
    
    
    /**
     * Hakee kuvaus-kentän sisällön
     * @return kuvaus
     */
    public String getKuvaus() {
        return this.kuvaus;
    }
    
    /**
     * Hakee avainsanat
     * @return Avainsanat-olio
     */
    public Avainsanat getAvainsanat() {
        return this.avainsanat;
    }

    
    /**
     * Asettaa otsikko-kentän
     * @param otsikko uusi otsikko
     */
    public void setOtsikko(String otsikko) {
        this.otsikko = otsikko;       
    }
    
    
    /**
     * Asettaa kuvaus-kentän
     * @param kuvaus uusi kuvaus
     */
    public void setKuvaus(String kuvaus) {
        this.kuvaus = kuvaus;
    }
   
    
    /**
     * Lukee tietueeseen tietoja jonosta " id [erotin] otsikko [erotin] avainsanat [erotin] kuvaus [erotin] muistiinpanot([erotin]...) "
     * Kenttiä luetaan vähintään kaksi ja annettuun lukumäärään asti, lopetetaan viimeistään neljännen kohdalla.
     * @param jono luettava jono
     * @param erotin merkki, jolla tiedot eroteltu 
     * @param kenttia kuinka monta kenttää luetaan
     * @return uusi Tietue. Null, jos id-arvoa ei saatu luettua tai kenttia on alle yksi.
     * @throws NumberFormatException ekaa kenttää ei saatu luettua ID:ksi
     * @throws IllegalArgumentException otsikkokenttää ei ole määritelty
     * @example
     * <pre name="test">
     *  Tietue t = Tietue.lueJono("1 | Otsikko | sana1, sana2, sana3 | Kuvaus | Muistiinpanot", '|', 5);
     *  t.toString() === "1 | Otsikko | sana1, sana2, sana3 | Kuvaus";  
     * </pre>
     */
    public static Tietue lueJono(String jono, char erotin, int kenttia) throws NumberFormatException, IllegalArgumentException {
        int luettavat = kenttia;
        if (luettavat < 2) luettavat = 2;
        StringBuilder sb = new StringBuilder(jono);
        return lueSB(sb, erotin, luettavat);
    }
    
    
    /**
     * Lukee Tietueeseen kentät StringBuilderista
     * @param sb StringBuilder
     * @param erotin kenttiä erottava merkki
     * @param kenttia montako kenttää luetaan
     * @return Tietueen kenttien perusteella, jos lukeminen onnistui
     * @throws NumberFormatException ekaa kenttää ei saatu luettua ID:ksi
     * @throws IllegalArgumentException otsikkokenttää ei ole määritelty
     * @example
     * <pre name="test">
     *  Tietue t;
     *  StringBuilder sb;
     *  sb = new StringBuilder($sb);
     *  t = Tietue.lueSB(sb, '|', $kenttia);
     *  t.toString() === $tulos;
     *  
     *  $sb                                      | $kenttia  | $tulos
     *  -------------------------------------------------------------------------------------------
     *  "1 | otsikko | sana1, sana2 | kuvaus"    |   4       | "1 | otsikko | sana1, sana2 | kuvaus"
     *  "1 | otsikko | s1, s2, s3, 4| kk |sd|fd" |   3       | "1 | otsikko | s1, s2, s3, 4 | "
     *  "1 | otsikko | s1, s2, s3, 4| kk |sd|fd" |   5       | "1 | otsikko | s1, s2, s3, 4 | kk"
     *  "0 | otsikko"                            |   5       | "0 | otsikko |  | "
     *  "6 | otsikko ||kuvaus|"                  |   4       | "6 | otsikko |  | kuvaus"
     *  
     *  try {
     *      t = Tietue.lueSB(new StringBuilder("a1 | otsikko"), '|', 4);
     *  } catch (NumberFormatException ie) {
     *      ie.getMessage() === "Ensimmäisen kentän arvo ei ole kokonaisluku: a1";
     *  }
     *  try {
     *      t = Tietue.lueSB(new StringBuilder("1 |    | sana1, sana2 | kuvaus"), '|', 4);
     *  } catch (IllegalArgumentException ie) {
     *      ie.getMessage() === "Merkkijonon otsikkokenttä on tyhjä";
     *  }
     * </pre>
     */
    public static Tietue lueSB(StringBuilder sb, char erotin, int kenttia) throws NumberFormatException, IllegalArgumentException {
        String s = Mjonot.erota(sb, erotin).strip();
        int id;
        try {
            id = Integer.parseInt(s);
        } catch (NumberFormatException ne) {
            throw new NumberFormatException("Ensimmäisen kentän arvo ei ole kokonaisluku: " + s);
        }
        s = Mjonot.erota(sb, erotin).strip();
        if (s.length() == 0) throw new IllegalArgumentException("Merkkijonon otsikkokenttä on tyhjä");
        Tietue uusi = new Tietue(id, s);
        if (kenttia > 2) uusi.getAvainsanat().lisaaJonosta(Mjonot.erota(sb, erotin).strip());
        if (kenttia > 3) uusi.setKuvaus(Mjonot.erota(sb, erotin).strip());
        return uusi;
    }
    
    
    /**Lukee jonon, jossa kentät on eroteltu merkillä '|'. Luetaan ensimmäiset neljä kenttää, jos ne löydetään.
     * @param jono luettava merkkijono
     * @return uuden Tietueen. Jos ensimmäistä kenttää ei saada luettua kokonaisluvuksi, null.
     */
    public static Tietue lueJono(String jono) {
        return lueJono(jono, '|', 4);
    }
    
    
    /**
     * Lukee jonon, jossa kentät on eroteltu merkillä '|'. Luetaan ensimmäiset neljä kenttää, jos ne löydetään.
     * @param jono luettava merkkijono
     * @param erotin merkki, jolla kentät erotellaan
     * @return uuden Tietueen. Jos ensimmäistä kenttää ei saada luettua kokonaisluvuksi, null.
     */
    public static Tietue lueJono(String jono, char erotin) {
        return lueJono(jono, erotin, 4);
    }
    
    
    /**
     * Lukee uuden Tietueen tiedot merkkijonotaulukosta: { ID, otsikko, avainsanat, kuvaus }
     * Taulukossa on oltava vähintään kaksi ensimmäistä alkiota, joista ensimmäinen on oltava
     * luettavissa kokonaisluvuksi. Ylimääräiset alkiot jätetään huomiotta.
     * @param taulukko luettava taulukko
     * @return uuden Tietueen. Jos luominen epäonnistuu, null.
     * @throws IllegalArgumentException Liian lyhyt taulukko
     * @throws NumberFormatException ID on oltava luettavissa kokonaisluvuksi
     * @example
     * <pre name="test">
     *  try {
     *      Tietue t = Tietue.lueTaulukko( new String[] { " 2 ", " otsikko    ", "    "});
     *      t.toString() === "2 | otsikko |  | ";
     *      t = Tietue.lueTaulukko( new String[] { " a2 ", " otsikko    ", "    "});
     *  } catch (NumberFormatException ne) {
     *    ne.toString() === "java.lang.NumberFormatException: For input string: \"a2\"";
     *  } catch (IllegalArgumentException ie) {
     *      System.out.println(ie);
     *  }
     *  try {
     *      Tietue t = Tietue.lueTaulukko( new String[] { " 2 ", " otsikko    ", " Sana1, Sana2   ", "kuvaus", "sälää", "Lisää sälää"});
     *      t.toString() === "2 | otsikko | Sana1, Sana2 | kuvaus";
     *      t = Tietue.lueTaulukko( new String[] { " a2 "});
     *  } catch (IllegalArgumentException ie) {
     *      ie.toString() === "java.lang.IllegalArgumentException: ID ja otsikko on oltava määritelty.";
     *  }
     *  
     *  try {
     *      Tietue t = Tietue.lueTaulukko( new String[] {});
     *  } catch (IllegalArgumentException ie) {
     *      ie.toString() === "java.lang.IllegalArgumentException: ID ja otsikko on oltava määritelty.";
     *  }
     * </pre>
     */
    public static Tietue lueTaulukko(String[] taulukko) throws IllegalArgumentException, NumberFormatException {
        if (taulukko.length < 2) {
            throw new IllegalArgumentException("ID ja otsikko on oltava määritelty.");
        }
        int id = Integer.parseInt(taulukko[0].strip());
        Tietue uusi = new Tietue(id, taulukko[1].strip());
        if (taulukko.length < 3) return uusi;
        uusi.getAvainsanat().lueJonosta(taulukko[2].strip());
        if (taulukko.length < 4) return uusi;
        uusi.setKuvaus(taulukko[3].strip());
        return uusi;
    }
    
    
    /**
     * Kopioi tiedot uuteen Tietueeseen.    
     * @return uuden Tietueen
     * @example
     * <pre name="test">
     *  Tietue t1 = Tietue.lueJono("12 | Testi | AS1,AS2 |  Kuvaus", '|');
     *  Tietue t2 = t1.kopioi();
     *  t1.onKopio(t2) === true;
     *  t1 = Tietue.lueJono("2 | Testi", '|');
     *  t2 = t1.kopioi();
     *  t2.onKopio(t1) === true;
     * </pre>
     */
    public Tietue kopioi() {        
        return new Tietue(this.id, this.otsikko, this.avainsanat, this.kuvaus);
    }
    
    
    /**
     * Tarkistaa, ovatko toisen tietueen tiedot samat kuin tämän
     * @param t toinen tietue
     * @return true, jos tiedot ovat samat
     *  Tietue t1 = Tietue.lueJono("12 | Testi | AS1,AS2 |  Kuvaus", '|');
     *  Tietue t2 = Tietue.lueJono("12 | Testi | AS1,AS2 |  Kuvaus", '|');
     *  Tietue t3 = Tietue.lueJono("12 | Testi | AS1,AS2,AS3 |  Kuvaus", '|');
     *  Tietue t4 = Tietue.lueJono("12 | Testi | AS1 |  Kuvaus", '|');
     *  Tietue t5 = Tietue.lueJono("12 | Testi | AS1,AS2 |  Kuaus", '|');
     *  Tietue t6 = Tietue.lueJono("12 | Testii | AS1,AS2 |  Kuvaus", '|');
     *  Tietue t7 = Tietue.lueJono("1 | Testi | AS1,AS2 |  Kuvaus", '|');
     *  Tietue t8 = Tietue.lueJono("2 | Testi", '|');
     *  Tietue t9 = Tietue.lueJono("2 | Testi", '|');
     *  t1.onKopio(t2) === true;
     *  t2.onKopio(t1) === true;
     *  t1.onKopio(t3) === false;
     *  t1.onKopio(t4) === false;
     *  t1.onKopio(t5) === false;
     *  t1.onKopio(t6) === false;
     *  t1.onKopio(t7) === false;
     *  t1.onKopio(t8) === false;
     *  t8.onKopio(t9) === true;
     */
    public boolean onKopio(Tietue t) {
        if (this.getID() != t.getID() || this.getKuvaus() != t.getKuvaus() || t.getOtsikko() != t.getOtsikko()) return false;
        if (this.avainsanat.getLkm() != t.avainsanat.getLkm()) return false;
        for (int i = 0; i < this.avainsanat.getLkm(); i++)
            if (this.avainsanat.get(i) != t.avainsanat.get(i)) return false;
        return true;
    }
    
    
    @Override
    public String toString() {
        return this.id + " | " + this.otsikko + " | " + this.avainsanat.toString() + " | " + this.kuvaus;
    }
    
}
